%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Apply ADMM to solve the Problem:
%          Min  -tr(F*Y)+ ro * ||Y||_* 
%          s.t. tr(X)=1, tensor(X) is supper-symmetric, X-Y=0
%
% Reference: "Tensor Principal Component Analysis via Convex Optimization"
%             Bo Jiang, Shiqian Ma, Shuzhong Zhang
%
% Available at:  http://arxiv.org/pdf/1212.2702v1.pdf
%
% Author: Bo Jiang and Shiqian Ma. 
% Last modified: July 11, 2013.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [X,val,iternum]=ADMPCA(F,X0,Y0,mu,ro,n,maxiter,p,m)

tol = 1.0e-6; N = n*n;

Lambda = eye(N,N); X = X0; Y = Y0; relerr = 1; iter = 1;

while (relerr > tol) && (iter < maxiter)

    iter = iter + 1;
    
    %%%%%%%%%%% Update X %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Xp = X;
    Z = Y + mu*Lambda; X = ProjSupSym(Z,n,p,m); X = (X + X')/2;
    
    %%%%%%%%%%%% Update Y %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Y = UpdateY(X,F,Lambda,mu,ro); Y = (Y + Y')/2;
    
    %%%%%%%%%%%% Update Lambda %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Lambda = Lambda - (X - Y)/mu; Lambda = (Lambda + Lambda')/2; 
    xerror = norm(X - Xp,'fro')/norm(Xp,'fro');
    xyerror = norm(X - Y,'fro');
 
    relerr = xerror + xyerror;
    
end

iternum=iter;
val = trace(F*X);