%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Problem: min ||X - Z||^2_F, st tr(X)=1, tensor(X) is supper-symmetric, 
% Input:   matrix Z \in \R^{n^2*n^2}
% Output:  matrix X, tensor(X) is supper-symmetric
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function X = ProjSupSym(Z,n,p,m)
%%%%%%%%% convert matrix Z to tensor Tz %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Tz = MatrixToTensor(Z,n);

%%%%%%%%% make Tz be supper-symmetric Fz %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Fz = sym_tensor(Tz,p,m);
%%%%%%%%% convert tensor Tz to matrix Z %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x = TensorToMatrix(Fz,n);


%%%%%%%%% calculate \sum_{i=1}^{n}Z_{(i-1)*n+i,(i-1)*n+i}%%%%%%%%%%%%%%%%%%
z1 = 0;
for i = 1:n
    z1 = z1+x((i-1)*n+i,(i-1)*n+i);
end

%%%%%%%%% calculate \sum_{u < v}\sum_{(ijkl)\in Permute(uuvv)}Z_{(i-1)*n+j,(k-1)*n+l}%%%%%%%%%%%%%%%%%%
z2 = 0;
for i = 1:n
    for j = i+1:n
        z2 = z2+6*x((i-1)*n+i,(j-1)*n+j);
    end
end
mu = (1/(n*(n+2)))*(3-3*z1-z2);

for i = 1:n
    x((i-1)*n+i,(i-1)*n+i) = mu+x((i-1)*n+i,(i-1)*n+i);
end

for i=1:n
    for j=i+1:n
        x((i-1)*n+i,(j-1)*n+j)=mu/3+x((i-1)*n+i,(j-1)*n+j);
        x((i-1)*n+j,(i-1)*n+j)=mu/3+x((i-1)*n+j,(i-1)*n+j);
        x((i-1)*n+j,(j-1)*n+i)=mu/3+x((i-1)*n+j,(j-1)*n+i);
        x((j-1)*n+i,(i-1)*n+j)=mu/3+x((j-1)*n+i,(i-1)*n+j);
        x((j-1)*n+i,(j-1)*n+i)=mu/3+x((j-1)*n+i,(j-1)*n+i);
        x((j-1)*n+j,(i-1)*n+i)=mu/3+x((j-1)*n+j,(i-1)*n+i);
    end
end


X = x;
