%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This is to solve a MRI example, which is Example 3.2 in the paper:
%             "Tensor Principal Component Analysis via Convex Optimization"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all; clc;

%% dimenstion of the problem
n = 3; d = 4;
%% parameters
iter = 2000; mu = 0.5; rho=10;

[p,m] = psort(d); % p is the set of all the permutations of (1,2,...,d) 


%% generate MRI example
F = zeros(3,3,3,3); 
F(1,1,1,1)=0.74694;
F(1,1,1,2)=-0.435103;
F(1,1,1,3)= 0.37089;
F(1,1,2,2)= 0.454945;
F(1,1,2,3)=-0.29883;
F(1,1,3,3)=1.24733;
F(1,2,2,2)=0.0657818;
F(1,2,2,3)=-0.795157;
F(1,2,3,3)=0.714359;
F(1,3,3,3)=-0.397391;
F(2,2,2,2)=1;
F(2,2,2,3)=0.139751;
F(2,2,3,3)=0.316264;
F(2,3,3,3)=-0.405544;
F(3,3,3,3)=0.794869;

F=sym_tensor(F,p,m) % make F super-symmetric
    
%% Call ADM to solve tensor PCA

X0 = zeros(n^2,n^2); X0(1,1) = 1; Y0 = zeros(n^2,n^2); Y0(n^2,n^2) = 1; % generate starting point

mF = TensorToMatrix(F,n); % transform F to be a matrix

% [X_ADM,v_ADM,iternum_ADM] = ADMPCA_SDP(mF,X0,Y0,mu,n,iter,p,m); % use ADM to solve tensor PCA with SDP relaxation

[X_ADM,v_ADM,iternum_ADM] = ADMPCA(mF,X0,Y0,mu,rho,n,iter,p,m) ; % use ADM to solve tensor PCA with nuclear norm penalty

[C,D]=eig(X_ADM);
index=find(abs(diag(D))>0.0001);
L=length(index);
if L==1
     fprintf('Global optimal solution has been identified. \n'); 
     X=C(:,index); % find the eigenvector with maximum eigenvalue
     Y=reshape(X,3,3);
     [y,x]=eig(Y);
     I=find(abs(diag(x))>0.0001);
     fprintf('\n');
     fprintf('The optimal solution is: \n');
     y(:,I)
end 




